/*
 * CachingWithAspectJ
 * Copyright (C) 2008 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.aspectj.caching;

import org.apache.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.christianschenk.aspectj.caching.helper.AspectJHelper;
import org.christianschenk.simplecache.SimpleCache;

/**
 * This aspect can be used to store objects in a cache. All you have to do is to annotate your
 * methods with <code>@Cachable</code>.
 * 
 * @author Christian Schenk
 */
@Aspect
public class CachingAspect {

	private static final Logger log = Logger.getLogger(CachingAspect.class);
	private final SimpleCache<Object> cache;

	public CachingAspect() {
		this.cache = new SimpleCache<Object>();
	}

	/**
	 * This pointcut matches all methods with a <code>@Cachable</code> annotation.
	 */
	@Pointcut("execution(@Cachable * *.*(..))")
	@SuppressWarnings("unused")
	private void cache() {
	}

	/**
	 * Returns objects from the cache if necessary.
	 */
	@Around("cache()")
	public Object aroundProfileMethods(final ProceedingJoinPoint thisJoinPoint) throws Throwable {
		final String thisJoinPointName = AspectJHelper.getJoinPointName(thisJoinPoint);
		final String thisJoinPointArgs = AspectJHelper.getJointPointArgs(thisJoinPoint);
		final String objName = thisJoinPointName + "-" + thisJoinPointArgs;
		log.debug("Executing method: " + objName);

		Object obj = this.cache.get(objName);
		if (obj == null) {
			obj = thisJoinPoint.proceed();
			this.cache.put(objName, obj);
		}
		return obj;
	}
}